#!/usr/bin/perl

# Document name: cgh_normalisation.pl
# cgh_normalisation.pl version 2.0
#
#
# Copyright (C) 2006 Johan Staaf

# cgh_normalisation.pl is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

# johan.staaf@med.lu.se
# Johan Staaf, Dept Oncology, Lund University, S-221 85 Lund, Sweden

use strict;
use FindBin ();



######## BASE file requirements #########
# The BASEfile format should be serial.
# expect per row: [position]	[reporter]	[chromosome] [startPosition]	[assay data]
# assayData should be in the form:
# [intensity1]	[intensity2]

# Missing values should have the form of Na or NaN

# R outputs following files:

# "plotfiles.txt" with the format:
# [plotfile1 assay 1]\t[plotfile2 assay2] ...
# [plotfile1 assay 2]\t ....
# from this file tar.bundled directories for each assay is created.



############# Accepted parameters sent to the script from the BASE plugin system #####

# The following parameters should be present, please observe that their exact name must
# be the same as listed below. Please make sure you know how to make plugins in BASE before 
# configuring a plugin.

	

############################################
############# MAIN  PROGRAM    #############
############################################


### Main Variables ###
my (@final);
my (%param_hash,%base_hash);
my (@cc,%assay_hash1,%assay_hash2);
my @assayRowsToRead=();
my @assayDirectoriesToCreate=();
my %assayPlotFilesToDirectory; #holds mapping of dir to which plotfiles
my %assayAnalysedFiles; #holds mapping of assay name to result data file from R for backreporting into BASE
my @cleanUp=();
#################

## Read the file and extract BASE column data #######

$/="\n\n"; #Gives every section as one lump
my $i=0;
my $fileIndex=0; #index of the number of assays to be analysed

while(<>){
	$i++;
	chomp;
	if($i==1){ #Parameters
		
		$_ =~ /filename\t(.+)\n/;
		$param_hash{'filename'}=$1;
		$_ =~ /excludeC\t(.+)\n/;  
		$param_hash{'excludeC'}=$1;
		
		$_ =~ /alpha\t(.+)\n/;  
		$param_hash{'alpha'}=$1;
		$_ =~ /mergeClusterMethod\t(.+)\n/;  
		$param_hash{'mergeClusterMethod'}=$1;
		$_ =~ /segMethod\t(.+)\n/;  
		$param_hash{'segMethod'}=$1;
		$_ =~ /lowessIter\t(\d+)\n/;  
		$param_hash{'lowessIter'}=$1;
		
		$_ =~ /tempWindowSel\t(\w+)\n/;  
		$param_hash{'tempWindowSel'}=$1;
		$_ =~ /plotLite\t(\w+)\n/;  
		$param_hash{'plotLite'}=$1;
		
		
		$_ =~ /constraint\t(.+)\n/;  
		$param_hash{'constraint'}=$1;
		$_ =~ /normMethod\t(\w+)\n/;  
		$param_hash{'normMethod'}=$1;
		$_ =~ /smooth\t(.+)\n/;  
		$param_hash{'smooth'}=$1;
		$_ =~ /noiseconstant\t(\d+)\n/;  
		$param_hash{'noiseconstant'}=$1;
		$_ =~ /lowessSpan\t(\d+\.?\d*)\n/; 
		$param_hash{'lowessSpan'}=$1;
		$_ =~ /lowessDelta\t(.+)\n/;  
		$param_hash{'lowessDelta'}=$1;
		$_ =~ /binSize\t(.+)\n/;  
		$param_hash{'binSize'}=$1;
		$_ =~ /quantileV\t(\d+\.?\d*)\n/;  
		$param_hash{'quantileV'}=$1;
		$_ =~ /method\t(\w+)\n/;  
		$param_hash{'method'}=$1;
		$_ =~ /outPutDiff\t(\w+)\n/;  
		$param_hash{'outPutDiff'}=$1;
		
		$_ =~ /mergeCluster\t(.+)\n/;  
		$param_hash{'mergeCluster'}=$1;
		
		$_ =~ /nbrClusters\t(\d)\n/;  
		$param_hash{'nbrClusters'}=$1;
		
		$_ =~ /maxC\t(.+)\n/;
		$param_hash{'maxC'}=$1;
			
		print"$_"; # re print directly to STDOUT!
		print"\n\n"; #to adjust!
	}elsif($i==2){ #section assays info
		$_=~ /count\t(\d*)/;
		$base_hash{'NumberOfAssays'}=$1;
		$_=~ /columns\t(.*)/;
		$base_hash{'AssayColumns'}=$1;
		$_=~ /annotationColumns\t(.*)/;
		$base_hash{'AnnotationColumns'}=$1;
		(my $text,my $assayInfo)=split("%",$_);
		$base_hash{'Assays'}=$assayInfo;
		
		my @assayInfo=split("\n",$base_hash{'Assays'});
		foreach my $line (@assayInfo){
			my @tmp=split("\t",$line);
			if($tmp[0] eq ""){
				#do nada, empty row
			}else{
				$assay_hash1{$tmp[1]}=$tmp[0]; #key = assay name , value = assay nbr
				$assay_hash2{$tmp[0]}=$tmp[1]; #key = assay nbr , value = assay name
			}
		}
		print"$_"; # re print directly to STDOUT!
		print"\n"; #to adjust!
	}else{ #assay info + section spots
		$_=~ /columns\t(.*)\tassayData/;
		$base_hash{'Columns'}=$1; #same for all spots sections
		$_=~ /channels\t(.*)/;
		$base_hash{'channels'}=$1; #same for all spots sections
		$_=~/assayFields\t(.*)/;
		$base_hash{'assayFields'}=$1; #same for all spots sections
		$_=~ /count\t(\d+)/;
		my $AssaySpecificCount = $1; #individual for each assay
		$_=~ /assays\t(\d+)/;
		my $AssaySpecificNbr = $1; #individual for each assay
				
		#### Make a header for the data files
		my $basematrixHeader=join("\t",$base_hash{'Columns'},$base_hash{'assayFields'});
				
		#### Create data vector for specific assay
		(my $text,my $data)=split("%",$_); #$data now holds the spot data!
		my @data=split("\n",$data);
		
		#### Print data to a separate file for each assay, with name = 'assayName'.txt
		$fileIndex++; #update by one!
		my $fileName="Sample".$fileIndex.".txt";
		push(@cleanUp,$fileName); #save of the data file names for later removal!
		
		open (myoutfile,">$fileName");
		print myoutfile "$basematrixHeader\n";
		my $c = 0;
		for(my $j=0;$j<@data;$j++){
			if($data[$j] eq ""){
				#do nada as empty row
			}else{
				# Expected format:
				# [position] [reporter] [chromosome] [startPosition] [intensity1] [intensity2]
				my @temp = split("\t",$data[$j]); #split each line
				if( ($temp[2]>0) && ($temp[3]>0) && ($temp[4]>0) && ($temp[5]>0) ){ #require a chromosome nbr >0, a positive startPos and positive intensities
					$c++;
					print myoutfile "$data[$j]\n";
				}
			}
		}#end for
		close myoutfile;
		
		#### Push the sample name to the Directory creation list!
		push(@assayDirectoriesToCreate,$assay_hash2{$AssaySpecificNbr});
				
		#### Collect information on which plot files goes into which directory
		my $plotFileName="Sample".$fileIndex."_plots.txt"; #guidance on which plotfiles goes into which directory
		$assayPlotFilesToDirectory{$assay_hash2{$AssaySpecificNbr}}=$plotFileName;
		# key = assayName = directory name, value = textfile name listing which plotfiles to collect!
		push(@cleanUp,$plotFileName); #save of the plot file names for later removal!
	
		#### Collect information on which plot files goes into which directory
		my $resultFileName="Analysed_Sample".$fileIndex.".txt"; #guidance on which result files goes with which assay for bioassay set construction
		$assayAnalysedFiles{$assay_hash2{$AssaySpecificNbr}}=$resultFileName;
		# key = assayName = directory name, value = Result data textfile for bioassay set construction
		push(@cleanUp,$resultFileName); #save of the plot file names for later removal!
	
		#### Add on to the vector telling how many rows to read for each assay 
		#### Format: [assayName]	[nbr of rows]	[fileName to read]	[plotFileName to collect plot file names to]
		push(@assayRowsToRead,join("\t",$assay_hash2{$AssaySpecificNbr},$c,$fileName,$plotFileName));
	}
} #end generating data text files

#### Print the assay to row file
open (myoutfile,">assayToRow.txt");
print myoutfile "Assay	Count	FileName	PlotFileName\n";
foreach my $line (@assayRowsToRead){
	print myoutfile "$line\n";
}
close myoutfile;
####

$/="\n"; #reset reading!

### Interpret the constraint parameter ##
# basically search the filter string for assay names and save
# them in vector

my %constraint;
if($param_hash{'constraint'} eq "NULL"){

}else{
	my @tmp=split(/\|/,$param_hash{'constraint'}); #split into vector, each assay
	foreach my $l (@tmp){ #foreach assay constraint
		if($l=~/\w/){
			my ($assay,$value)=split(":",$l);
			#print"#$assay#$value#\n";
			$assay="\"".$assay."\"";
			$constraint{$assay}=$value;
		}
	}
}
####


#### OBSERVE, all base headers now printed to STDOUT!


#### Write parameters to R run file
#if($param_hash{'filename'} eq "cgh_normalisation.R"){
	#open(myfile,"<$FindBin::Bin/cgh_normalisation.R") or die "can't locate $param_hash{'filename'} file!";
	open(myfile,"<$FindBin::Bin/$param_hash{'filename'}") or die "can't locate $param_hash{'filename'} file!";
	
	open (myoutfile, ">cgh_normalisationCommands.R");
	while(<myfile>){
		chomp;
		
		if($_ =~ /##normMethod##/){
			print myoutfile "normMethod<-\"$param_hash{'normMethod'}\"\n";
			next;
      		}
		if($_ =~ /##constraint##/){
      			if($param_hash{'constraint'} eq "NULL"){
				print myoutfile "constraintA<-\"NULL\"\n";
				print myoutfile "constraintV<-\"NULL\"\n";
			}else{
	      			my $cA=join(",",keys(%constraint));
	      			my $cV=join(",",values(%constraint));
	      			$cA="c(".$cA.")";
	      			$cV="c(".$cV.")";
	      			#print "$cA\n$cV\n";
				print myoutfile "constraintA<-$cA\n";
				print myoutfile "constraintV<-$cV\n";
			}
			next;
      		}
      		if($_ =~ /##smooth##/){
			print myoutfile "smooth<-$param_hash{'smooth'}\n";
			next;
      		}
		if($_ =~ /##cytobandPath##/){
			print myoutfile "cytoband<-read.table(\"$FindBin::Bin/cytoBand.txt\",header=FALSE,na.strings=c(NA))\n";
			next;
		}      		
      		if($_ =~ /##maxC##/){
			print myoutfile "maxC<-$param_hash{'maxC'}\n";
			next;
      		}
      		if($_ =~ /##quantileV##/){
			print myoutfile "quantileV<-$param_hash{'quantileV'}\n";
			next;
      		}
      		if($_ =~ /##noiseconstant##/){
			print myoutfile "noiseconstant<-$param_hash{'noiseconstant'}\n";
			next;
      		}
		if($_ =~ /##lowessSpan##/){
			print myoutfile "lowessSpan<-$param_hash{'lowessSpan'}\n";
			next;
      		}
      		if($_ =~ /##nbrClusters##/){
			print myoutfile "nbrClusters<-$param_hash{'nbrClusters'}\n";
			next;
      		}
      		if($_ =~ /##lowessIter##/){
			print myoutfile "lowessIter<-$param_hash{'lowessIter'}\n";
			next;
      		}
      		if($_ =~ /##lowessDelta##/){
			print myoutfile "lowessDelta<-$param_hash{'lowessDelta'}\n";
			next;
      		}
		if($_ =~ /##excludeC##/){
			print myoutfile "excludeC<-$param_hash{'excludeC'}\n";
			next;
      		}
      		if($_ =~ /##binSize##/){
			print myoutfile "binSize<-$param_hash{'binSize'}\n";
			next;
      		}
		if($_ =~ /##outPutDiff##/){
			print myoutfile "outPutDiff<-$param_hash{'outPutDiff'}\n";
			next;
      		}
      		if($_ =~ /##mergeCluster##/){
			print myoutfile "mergeCluster<-$param_hash{'mergeCluster'}\n";
			next;
      		}      		
      		if($_ =~ /##alpha##/){
			print myoutfile "alpha<-$param_hash{'alpha'}\n";
			next;
      		}  
      		if($_ =~ /##plotLite##/){
			print myoutfile "plotLite<-$param_hash{'plotLite'}\n";
			next;
      		}  
      		if($_ =~ /##method##/){
			print myoutfile "method<-\"$param_hash{'method'}\"\n";
			next;
      		}
      		if($_ =~ /##segMethod##/){
			print myoutfile "segMethod<-\"$param_hash{'segMethod'}\"\n";
			next;
      		}
      		if($_ =~ /##mergeClusterMethod##/){
			print myoutfile "mergeClusterMethod<-\"$param_hash{'mergeClusterMethod'}\"\n";
			next;
      		}

		if($_ =~ /##tempWindowSel##/){
			print myoutfile "tempWindowSel<-\"$param_hash{'tempWindowSel'}\"\n";
			next;
      		}      		
      		
      		
      		
      		print myoutfile "$_\n";
	} #end of while
	close (myfile);
	close (myoutfile);
#} #end  if(filename eq)
	


##### Launch R #####
my $success = system("R --no-save < cgh_normalisationCommands.R > r-debug.txt");

#if($success != 0){
#	open(myfile,">ErrorMessage.txt");
#	print myfile "R session did not exit nicely!\n";
#	print myfile "There may be several reasons for this. A major reason could be that you have to few\n";
#	print myfile "clones per chromosome left after smooth and removal of missing values. This could result in\n";
#	print myfile "the clustering algorithm not being able to find correct cluster centers. Try to set the minimum \n";
#	print myfile "number of clones higher, or to increase the number of iterations for the clustering. The first\n";
#	print myfile "alternative is more likely to work. :)\n";
#	close(myfile);
#	die; #terminate plugin
#}	
# system starts process and waits for R to complete

#### Process output

#if($param_hash{'filename'} eq "cgh_normalisation.R"){ #process som output file
	# Return to base a serial BASE file with dummy positions
	
	# output file is printed as 
	# [position]	[reporter]	[intensity1]	[intensity2]
	
	#section	assays
	#count	35
	#columns	id	name	population
	#annotationColumns	population
	#%
	### Corresponding already printed
		
	$/="\n"; #resetting just in case!
		
	foreach my $assayQ (@assayDirectoriesToCreate){
		#open(myfile,"cghanalysed.txt") or die "can't locate cghanalysed file!"; #open result file		
		open(myfile,$assayAnalysedFiles{$assayQ}) or die "can't locate $assayAnalysedFiles{$assayQ} file!"; #open result file for selected assay
		print "\nsection	spots\n";
		print "columns\tposition\treporter\tassayData\n";
		print "channels\t$base_hash{'channels'}\n";
		print "assayFields\tintensity1\tintensity2\n";
		print "assays	$assay_hash1{$assayQ}\n";
		print "%\n";
		
		## Data	lines
		while(<myfile>){
			chomp;
			if($_ =~/e/){
				my @tmp=split("\t",$_);
				foreach my $ll (@tmp){
					if($ll =~/(\d+\.?\d+)e\+(\d+)/){
						$ll=$1*10**$2;
					}elsif($ll =~/(\d+\.?\d+)e-(\d+)/){
						$ll=$1/10**$2;
					}
				}
				my $line=join("\t",@tmp);
				print "$line\n";
			}else{
				print "$_\n";
			}
		} 
		close myfile;
	} # end of creation of BASE bioassay set
	
	
	## Post processing files steps
	#	1. Create the directory structure
	#	2. Move respective assays plots to its directory
	#	3. Remove temporary files
	#	4. Create HTML Reports
		
	# 1 # Create directories
	foreach my $line (@assayDirectoriesToCreate){
		my $commandline= "mkdir $line";
		system("$commandline >dump.txt");
	}
	
			
	# 2 # Move assay plots to their directories
	foreach my $assay (@assayDirectoriesToCreate){
		open(myfile,$assayPlotFilesToDirectory{$assay}); #open corresponding plotfile file name summary
		while(<myfile>){
			chomp;
			if($_ ne ""){
				my $commandline="mv $_ $assay";
				system("$commandline >dump.txt"); #move files to right directory
			}
		}
		close(myfile);
	}
	
	# 3 # Clean up, remove all the temporary assayData files once used
	if($success eq 0){
		#R exited nicely, remove temporary files
		foreach my $file (@cleanUp){
			my $commandline = "rm $file";
			system("$commandline >dump.txt");
		}	
	}
	
	#### Create HTML summary tables
	my @fileVector=("CorrectionSummary.txt"); 
	my @htmlFileVector=("CorrectionFactorSummary.html");
	my @htmlTitle=("Correction Factor Summary");
	my @htmlH1=("Correction Factor Summary per assay");
	
	for (my $e=0;$e<@fileVector;$e++){
		my $file=$fileVector[$e];
		my $myhtml=$htmlFileVector[$e];
		my @table=();
				
		open(myfile,"$file");
		while(<myfile>){
			chomp;
			push(@table,$_);
		} #end of while
		close(myfile);
			
		#@table now holds all information
		my @header=split("\t",shift(@table));
	
		open(myhtml,">$myhtml");
		print myhtml "<HTML>\n<HEAD>\n<TITLE>$htmlTitle[$e]</TITLE>\n";
		print myhtml "<style type=\"text/css\">\n";
		print myhtml "body { font-family: Helvetica, sans-serif; background-color: #F8F8F0; font-size: 9pt; font-weight:  normal }\n";
		print myhtml "table { cellpadding: 1; cellspacing: 2; border: 1 }\n";
		print myhtml "td { font-family: Helvetica, sans-serif; font-size: 9pt }\n";
		print myhtml "th { font-family: Helvetica, sans-serif;  text-align: left; background-color: #D3D3CC; font-size: 9pt }\n";
		print myhtml "tr.dark { background-color: #cecece }\n";
		print myhtml "tr.light { background-color: #e0e0e0 }\n";
		print myhtml ".pagehead { text-align: left; background-color: #F8F8F0; font-size: 11pt }\n";
		print myhtml ".subhead { text-align: left; font-size: 10pt; font-weight: bold }\n";
		print myhtml ".special { font-weight: bold }\n";
		print myhtml "</style></HEAD><BODY><H1><center>$htmlH1[$e]</center></H1><br>\n";
		print myhtml "<TABLE>\n";
		print myhtml "<tr>";	
		foreach my $headeritem (@header){
			print myhtml "<th><center>$headeritem</center></th>";
		}
		print myhtml "</tr>"; #header printed
					
		my $t=0;
		foreach my $line (@table){ 
			$t++;
			my @temp=split("\t",$line);
			print myhtml "<tr>"; #new row
			for(my $cell=0;$cell<@temp;$cell++){
				if($t%2){		
					print myhtml "<td bgcolor = \"#f0f0f0\"><center>$temp[$cell]</center></td>";
				}else{
					print myhtml "<td bgcolor = \"white\"><center>$temp[$cell]</center></td>";
				}
			}
			print myhtml "</tr>";
		}#end of foreach
		print myhtml "</TABLE>";
		print myhtml "\n</BODY>\n</HTML>\n";
		close myhtml;
	}#end for each file
#} #end of if param_hash


##### General Clean up #######
#system("rm assayToRow.txt") ==0 or die "cleanup of temporary files failed ($!)!";
#system("rm cgh_normalisationCommands.R") == 0 or die "cleanup of temporary files failed ($!)!";
system("rm dump.txt");
#system("rm CorrectionSummary.txt");