
Compute_kmean_and_levels<-function(x,noiseconstant,init.point,max.iterations,percent){
	# performs three means clustering on expected log2(ratios) for one chromosome at
	# a time. Please note that the vector x passed should not contain missing values.
	
	# cluster3 is the top cluster, amp
	# cluster2 is the middle cluster, unchanged
	# cluster1 is the lower cluster, del
	# a number of safety checks are added:
	# 1. If less than 50 real data values, nbr is set to 1. 
	# 2. If nbrBP >= length(x), then nbrBP is set to length(x)-5 . This might not be the
	# perfect solution.
	
	# Argument:
	# x is a dataframe / vector containing log2(ratio) for one assay
	# noiseconstant is an integer related to how noisy the data is. Increasing the noiseconstant
	# increases the number of breakpoints per chromosome.
	# init.point is an integer specifying which largest / smallest value to use as initial clusterpoints
	# max.iterations is an integer specifying the maximal number of iterations for the kmean clustering
	
	# Value:
	# an integer,nbrBP, specifying the number of breakpoints along the investigated chromosome.
	# Corresponds to c in the published paper.
	
	# Parameters, check how to put them as default ??
	
	#### Removing any missing values ####
	Nainds<-which(is.na(x)==FALSE)
	x<-x[Nainds]
	
	#### initialization
	nbrOfValues<-length(x)
	nbrClustersToUse<-3
	if(nbrOfValues<nbrClustersToUse){
		#less than e.g. 3 data points!
		return(1)
	}
	firstsort<-sort(x,index.return=TRUE) #sorting lowest first, largest last index
	cluster2.point<-median(x)
	cluster1.point<-x[firstsort$ix[init.point]] #fitht smallest if init.point = 5
	cluster3.point<-x[firstsort$ix[(nbrOfValues-init.point)]] #fitht largest if init.point = 5
	
	if(length(which(x==cluster2.point))==0){
		#cluster point 2 is not a real data point! Can cause problems, assign a real one close to center!
		cluster2.point<-x[firstsort$ix[round(length(x)/2)]]
	}
	
	## Cluster the data	
	
	if(length(unique(c(cluster1.point,cluster2.point,cluster3.point)))<3){
		print("not unique cluster suggestions, suggested:")
		print(c(cluster1.point,cluster2.point,cluster3.point))
		#TAKE actions???
		# try a lower init.pt in order to find better outer clones init 1 and 2
		my.succeded<-FALSE
		for(w in 1:10){ #10 attempts
			new.init<-init.point-w
			if(new.init<1){
				break
			}else{
				cluster1.point<-x[firstsort$ix[new.init]] #
				cluster3.point<-x[firstsort$ix[(nbrOfValues-new.init)]] #
				
				if(length(unique(c(cluster1.point,cluster2.point,cluster3.point)))==3){
					#found three unique init points
					my.succeded<-TRUE
					break #exit for loop
				}
			}
		}
		if(!my.succeded){
			#if init.point approach failed...
			print("still no success")
			return(1)
			
		}
	}
	rm(firstsort)
	
	# The data given by x is clustered by the k-means algorithm. When this terminates, 
	# all cluster centres are at the mean of their Voronoi sets (the set of data points which 
	# are nearest to the cluster centre). The algorithm of Hartigan and Wong (1979) is used. 
	# centers Either the number of clusters or a set of initial cluster centers. 
	# If the first, a random set of rows in x are chosen as the initial centers.  
	
	#cl <- kmeans(x, 3, max.iterations)  #random rows used
	cl <- kmeans(x, c(cluster1.point,cluster2.point,cluster3.point), max.iterations) #defined initial centers used
	
	## Calculate the number of breakpoints
	cl3.sort<-sort(x[which(cl$cluster==3)]) #lowest comes first
	cl1.sort<-sort(x[which(cl$cluster==1)])
	
	nbr<-trunc(percent*length(x)) # percent of all clones in the chromosome, defining e.g. top 2%
	
	if(nbr==0){ # if less than e.g. 50 values (using percent = 0.02) in total for a chromosome, set nbr = 1
		nbr<-1
		print("put nbr to = 1 due to few clones in chromosome")
	}
		
	if( length(cl3.sort)>(length(x) - nbr) ){
		# more than 100 - percent (e.g. 2) of all reporters in one of the amp/del clusters
		# meaning changes = 0
		nbrBP<-0
	}else{
		if (length(cl1.sort)>(length(x) - nbr) ){
			# more than 100 - percent (e.g. 2) of all reporters in one of the amp/del clusters
			# meaning changes = 0
			nbrBP<-0
		}else{
			#### The alternative described in the paper
			
			#nbr3<-trunc(length(cl3.sort)*(1-percent))
			#nbr1<-trunc(percent*length(cl1.sort))
			
			#cl3.m<-mean(cl3.sort[nbr3:length(cl3.sort)] )
			#cl1.m<-mean(cl1.sort[1:nbr1])
	
			#nbrBP<-abs(trunc( noiseconstant*(cl3.m - cl1.m) ) )
	     		#####
	
	
			#### The alternative used in the MATLAB code
			
			if(length(cl3.sort)<nbr){
				#to few reporters in cluster 1, add from cluster 2
				cl3.expanded<-c(cl3.sort,x[which(cl$cluster==2)])
				cl3.expanded<-sort(cl3.expanded)
				print(paste("length(cl3.expanded) ", length(cl3.expanded)))
				cl3.m<-mean(cl3.expanded[nbr:length(cl3.expanded)] )
			}else{
				cl3.m<-mean(cl3.sort[nbr:length(cl3.sort)] )
			}
			if(length(cl1.sort)<nbr){
				#to few reporters in cluster -1, add from cluster 0
				cl1.expanded<-c(cl1.sort,x[which(cl$cluster==2)])
				cl1.expanded<-sort(cl1.expanded)
				cl1.m<-mean(cl1.expanded[1:nbr] )
			}else{
				cl1.m<-mean(cl1.sort[1:nbr])
			}
			nbrBP<-abs(trunc( noiseconstant*( cl3.m-cl1.m ) ) )
			#####
		}
	}
	
	### Safety checks ###
	if(nbrBP>=nbrOfValues){
		nbrBP<-nbrOfValues-10
		print("nbrBP set to length(x) - 10!")
	}
	if(nbrBP<0){
		nbrBP<-0
	}
	###
	
	return(nbrBP)
}


Dynamic_prog<-function(x,c){
	# Function 'DynamicProg' calculates constant levels for the CGH-data using Dynamic Programming. 
	# It is assumed that CGH-signal can be approximated with a constant and an error, and limited 
	# amount of these constant levels can be identified in same chromosome. These levels usually 
	# span over larger region of chromosome. If the number of changes in constant levels is c, 
	# data consists of c+1 constant levels.  
	#	         
	# Reference:  Steven M. Kay: Fundamentals of Statistical signal processing
	#             Detecting Theory           
	# IN:      
	#      cgh_data      cgh-data, consisting of kb sorted log2(ratios) from one assay and one chromosome at a time
	#  	      	     e.g. chromosome 6 assay 1. Could include NaNslog2(ratio) 
	#      c             number of chances between constant levels
	#                
	#               
	# OUT:
	#      A list with components:      
	#	step      : log2(ratio) values for the constant levels of the CGH-data 
	#       levelstart     : the start index for each level (< index!) e.g. 3,5 means that index 1 & 2 is in the
	#  		       first level as 2<3 and index 3 &4 is in the second level as 4<5 etc. Length = C 
	#	levelend	:the end index for each level
	
	lz<-list()
			
	# Size of the data
	N<-length(x)
	
	if(c==0){ # NO BREAKPOINTS WHAT SO EVER!!
		lz$step<- matrix(nrow=1,rep(mean(x,na.rm=TRUE),N))
		lz$levelstart<-1
		lz$levelend<-N
		return(lz)
	}
	if(N==1){
		lz$step<- matrix(nrow=1,ncol=1,x)
		lz$levelstart<-1
		lz$levelend<-N
		return(lz)				
	}
	
	# I is the (datapoint - mean value of the constant level)^2
	# Computed assuming that each point is the last one of first constant level
	
	# Initialize
	md<-cumsum(x)/(1:N)
	I<-matrix(ncol=N,nrow=c,0) 
	
	I[1,]<-cumsum(x^2) + (1:N)*md^2 - 2*cumsum(x)*md
	J<-NULL
	
	
	ntrans<-matrix(ncol=c,nrow=N,0) #columns = number of levels , #rows number of clones
	
	#starting to check c steps
	
	for (k in 1:c){  # Check followed steps
	    if (k<c) {
	    	for (L in k:(N-c+k)){           
	            # n defines from which point, L where to
	            # Prepares the matrix J. Makes sure that any value of J, which doesn't chance, is not the min value of J.
	            
	            J[1:k]<-10000
	            
	            md<-cumsum(rev(x[k:L]))/(1:(L-k+1))
	            # Del defines the sum of error between datapoints and constant level. 
	            Del<-cumsum(rev(x[k:L])^2) + (1:(L-k+1))*md^2 - 2*cumsum(rev(x[k:L]))*md
	            
	            # J(n+1)= the sum of errors of last steps plus error of next one.
	            J[(k+1):(L+1)]<-I[k,(k:L)]+rev(Del)
	            	            
	            # Pick up the min value of J for each k 
	            # I gets the value, ntrans tells the row where the min(J(1:LL)) is found 
	            # ntrans tells the best path to go point L                                           
	            	            
	            I[(k+1),L]<-min(J[1:L])
	            ntrans[L,k]<-which.min(J[1:L])          
	        }
	    }else{
	        L=N # last step
	        # Prepares J
	        J[1:k]<-10000 
	        # Checking all possible n, from where the next step could be N                                
	        md<-cumsum(rev(x[k:L]))/(1:(L-k+1))
	        Del<-cumsum(rev(x[k:L])^2) + (1:(L-k+1))*md^2 - 2*cumsum(rev(x[k:L]))*md
	        J[(k+1):(L+1)]<-I[k,(k:L)]+rev(Del)
	        # Min value of J
	        ntrans[N,k]<-which.min(J[1:N])
	        Imin<-min(J[1:L])       
	    }
	}
	
	# Define estimation values for steps
	# beginning of the last constant level
	tul<-matrix(nrow=1,ncol=c,0)
	tul[1,c]<- ntrans[N,c]
		
	if(c>1){
		for (index in 1:(c-1)){
			if((tul[1,(c-index+1)]-1)<1){
				#Error, index of 0 indicated!
				#break and return the chromosome arm as a
				#single long segment.
				lz$step<- matrix(nrow=1,rep(median(x,na.rm=TRUE),N))
				lz$levelstart<-1
				lz$levelend<-N
				print("Ended dynamic prog due to index error!!")
				return(lz)
		    	}else{	
		    		tul[1,(c-index)]<-ntrans[(tul[1,(c-index+1)]-1),(c-index)]   #finding the previous step
			}
		}
		tul<-tul-1
		Tulos<-cbind(1,tul,(N+1))
		# draws askel function from previous step to next one and  
		# pictures every datapoint to the constant level (steps average)  
		step <-matrix(nrow=1,ncol=0) #?
	
		for (ii in 1:(length(Tulos)-1)){
		    if (Tulos[ii] == Tulos[ii+1]){
		        # if the first value of the chromosome is a change point nothing happens,
		        # because first value always starts new constant level (otherwise impossible)
		    }else{
		        step <- cbind(step,(mean((x[Tulos[1,ii]:(Tulos[1,(ii+1)]-1)]))*matrix(nrow=1,ncol=length(x[Tulos[1,ii]:(Tulos[1,(ii+1)]-1)]),1) ) )
		    	#step <- cbind(step,(mean(2^(x[Tulos[1,ii]:(Tulos[1,(ii+1)]-1)]) )*matrix(nrow=1,ncol=length(x[Tulos[1,ii]:(Tulos[1,(ii+1)]-1)]),1) ) )
		    }    
		} # for i in 1:length(Tulos)-1
	}else{
		if(c==1){ #only one breakpoint
			tul<-tul-1
			Tulos<-cbind(1,tul[1,c],(N+1))
			# draws askel function from previous step to next one and  
			# pictures every datapoint to the constant level (steps average)  
			step <-matrix(nrow=1,ncol=0) #?
				
			for (ii in 1:(length(Tulos)-1)){
			    if (Tulos[ii] == Tulos[ii+1]){
			        # if the first value of the chromosome is a change point nothing happens,
			        # because first value always starts new constant level (otherwise impossible)
			    }else{
			        step <- cbind(step,(mean((x[Tulos[1,ii]:(Tulos[1,(ii+1)]-1)]))*matrix(nrow=1,ncol=length(x[Tulos[1,ii]:(Tulos[1,(ii+1)]-1)]),1) ) )
			    }    
			} # for i in 1:length(Tulos)-1
		}
	}
	lz$step<- step
	lz$levelstart<-c(1,tul)
	lz$levelend<-c(tul-1,length(step))
	
	####
	rm(x)
	return(lz)
}



########################### MAIN ###############################33

### Read the file with assays to process ####
#   file name should be 'assayToRow.txt'
#   format should be:
#   Assay Count      FileName        PlotFileName
my.colClasses<-c("character","integer","character","character")
assaysToRow<-read.table("assayToRow.txt",colClasses=my.colClasses,header=TRUE,na.strings=c(NA))
nbrAssays<-length(assaysToRow$Assay)
###

### Read Cytobands ###

##cytobandPath##

#cytoband<-read.table("cytoBand.txt",header=FALSE,na.strings=c(NA))
colnames(cytoband)<-c("chromosome","start","end","name","plotstatus")
cytoband$name<-as.character(cytoband$name)
cytoband$plotstatus<-as.character(cytoband$plotstatus)
cytoband$color<-rep("",length(cytoband$name)) #holder of plotcolor

my.blacks<-grep("gpos\d*",cytoband$plotstatus)
my.whites<-grep("gneg\d*",cytoband$plotstatus)
my.gvar<-grep("gvar\d*",cytoband$plotstatus)
my.acen<-grep("acen\d*",cytoband$plotstatus)
my.stalk<-grep("stalk\d*",cytoband$plotstatus)
cytoband$color[my.stalk]<-"white"
cytoband$color[my.acen]<-"black"
cytoband$color[my.blacks]<-"black"
cytoband$color[my.gvar]<-"black"
cytoband$color[my.whites]<-"white"
######



## Required for lognormal estimation ##
library(MASS)
##

### Parameters ###

##excludeC##
##binSize##
##smooth##
##constraint##
##normMethod##
##lowessSpan##
##noiseconstant##
##quantileV##
##method##
##mergeCluster##
##maxC##
##lowessDelta##
##segMethod##
##mergeClusterMethod##
##lowessIter##
##nbrClusters##
##tempWindowSel##


## Find centromer limits
cyto.chromosomes<-seq(1,24)
centromer.limit<-rep(0,length(cyto.chromosomes))
for(j in 1:length(centromer.limit)){
	cChr<-cyto.chromosomes[j]
	if(cChr==23){ # X chrom
		cChr<-"X"
	}
	if(cChr==24){ # Y chrom
		cChr<-"Y"
	}
	cChr<-paste("chr",cChr,sep="")
	c.Chr.index<-which(cytoband$chromosome==cChr)
	cytoChr<-cytoband[c.Chr.index,] #index 5 is the plotstatus, 6, the plot color
	centromer.limit[j]<-cytoChr$end[max(grep("p",cytoChr$name))]
	rm(cytoChr,cChr,c.Chr.index)
}

	
### Define a vector for correction factor summary and for DLR summary##
ReporterNbrSummary<-vector(mode = "numeric", length =nbrAssays )

### Define a vector for correction factor summary and for DLR summary##
CorrectionSummary<-matrix(nrow=nbrAssays,ncol=2,0) #col 1 = old mean(M), col2=new mean M

for(i in 1:nbrAssays){  
	#### Make a structure for holding plot file names
	my.plotfiles<-matrix(nrow=8,ncol=1,"") #match to nbr of plots below!!
	
	smoothSD<-0 #assume
	
	# Read data for a specific assay
	# expect format:
	# position	reporter	chromosome	startPosition	intensity1	intensity2
	assayName<-assaysToRow$Assay[i]
	print(assayName)
	
	my.colClasses<-c("integer","integer","character","integer","numeric","numeric")
	data<-read.table(assaysToRow$FileName[i],header=TRUE,colClasses=my.colClasses,nrows=assaysToRow$Count[i],na.strings=c(NA))
	my.col.names<-colnames(data)
	data$l2ratio1_2<-log2(data$intensity1/data$intensity2)
	data$l10intgmean1_2<-log10(sqrt(data$intensity1*data$intensity2))	#A, log10(sqrt(Ch1*Ch2))
		
	# If using X and Y characters instead of numbers..
	data$chromosome[which(data$chromosome == "Y")]<-24 #x=23,y=24
	data$chromosome[which(data$chromosome == "X")]<-23
	data$chromosome<-as.integer(data$chromosome)
	#
		
	chromosomes<-sort(unique(data$chromosome))
		
	#### sort the data according to chromosome and startPosition
	ii<-order(data$chromosome,data$startPosition)
	data<-data[ii,] #reorder all data according to chromosom and then to start kb
	rm(ii)
	gc();
	PosOrder<-seq(1,length(data$chromosome))
		
	## (Externally) find the highest posOrder foreach chromosome 
	chr.MaxPosOrder<-vector(mode = "integer", length = length(chromosomes))
	for(j in 1:length(chromosomes)){
		chr.MaxPosOrder[j]<-max(which(data$chromosome==chromosomes[j]),na.rm=TRUE)
	}
		
		
	#### Exclude options (exclude chromosomes from calculation, however do not filter them out)
	ii<-match(excludeC,chromosomes)
	chromosomesC<-vector()
	if(length(ii[which(!is.na(ii))])>0){
		chromosomesC<-chromosomes[-c(ii)]	
	}else{
		chromosomesC<-chromosomes
	}


	#### Do the SD smoothing
	SDvector<-vector() #collects the window SD:s
	SDrepvector<-vector() #to collect positions
	for(c in 1:length(chromosomesC)){
		workingChrom<-chromosomesC[c]
		data.subset<-data[which(data$chromosome==workingChrom),]
		r<-1:length(data.subset$l2ratio1_2)
		
		##assuming smooth is a percentage of clones residing on a chromosome
		smooth.size<-round(smooth*length(data.subset$l2ratio1_2))
		
		if(tempWindowSel=="clones"){
			#temp: WINDOW defined by clones, i.e. smooth = integer!
			smooth.size<-smooth
		}		
		
		print(c(c,smooth.size))
		
		r.start<-r-smooth.size
		r.end<-r+smooth.size
		
		#trimming the vectors
		ii<-seq(1+smooth.size,length(r)-smooth.size)
		r.run<-r[ii]
		r.start<-r.start[ii]
		r.end<-r.end[ii]
		
		localV<-rep(0,length(r.run))
		localR<-rep(0,length(r.run))
		for(u in 1:length(r.run)){
			local.sd<-sd(data.subset$l2ratio1_2[r.start[u]:r.end[u]],na.rm=TRUE)
			localV[u]<-local.sd
			localR[u]<-data.subset$position[u]
		}
		SDvector<-c(SDvector,localV)
		SDrepvector<-c(SDrepvector,localR)
		rm(localV,localR,r.start,r.end,r.run,ii,r,data.subset)
	}#end smoothing
	
	
	### clear any NA:s introduced..
	na.indexes<-which(is.na(SDvector))
	if(length(na.indexes)>0){
		SDvector<-SDvector[-na.indexes]
		SDrepvector<-SDrepvector[-na.indexes]
	}
	rm(na.indexes,local.sd)	
	gc();
	###
	

	### Collect boxplot SD data for specific assay
	v<-boxplot.stats(SDvector,coef = 1.5)
	###
	
	ff.sd<-density(SDvector, kernel="gaussian", bw=0.01) #estimate kernel function
	kernel.sd<-ff.sd$x[which.max(ff.sd$y)]
		
	print(paste("smoothSD median:",median(SDvector)))
	print(paste("kernel SD:",kernel.sd))
	
	## estimates an SD based on drop off peak curve to 0.607 of peak height.
	u.d<-ff.sd$y[which.max(ff.sd$y)]
	u.1sd<-0.607*u.d #the height at 1 sd for a normal distribution with mean u. 
	lower<-ff.sd$y[which(ff.sd$x<kernel.sd)]
	upper<-ff.sd$y[which(ff.sd$x>kernel.sd)]
	lowD<-max(lower[which(lower<=u.1sd)])
	highD<-max(upper[which(upper<=u.1sd)])
	lowSD<-ff.sd$x[which(ff.sd$y==lowD)]
	highSD<-ff.sd$x[which(ff.sd$y==highD)]
	my.sd<-mean(c(kernel.sd-lowSD,highSD-kernel.sd))
	#my.sd<-min(c(kernel.sd-lowSD,highSD-kernel.sd))
	
	rm(u.d,u.1sd,lower,upper,lowD,highD,lowSD,highSD)
		
	myfile<-paste(assayName,"_SDplot.ps",sep="")
	my.plotfiles[1,1]<-myfile
	postscript(myfile)
	my.segBreaks<-round( ( max(SDvector,na.rm=TRUE)-min(SDvector,na.rm=TRUE))/(1*binSize))
	vvv<-hist(SDvector,breaks=my.segBreaks,plot=FALSE)
	my.ylim<-c(0,max(vvv$density)+3)
	rm(vvv)
	op <- par(omd=c(0,1,0,0.93)) #increase the outer margins slightly		
	hist(SDvector,breaks=my.segBreaks,freq=FALSE,ylim=my.ylim,las=1,xaxt="n",cex.main=0.8,main="",xlab="sd")
	title(main="Histogram of SDvector",outer=TRUE,cex.main=0.9)
	axis(side=1,at=seq(0,round(max(SDvector,na.rm=TRUE)),by=0.1),labels=seq(0,round(max(SDvector,na.rm=TRUE)),by=0.1),cex.axis=0.7)
	lines(ff.sd,col="red")
	curve((dnorm(x,sd=my.sd,mean=kernel.sd, log=FALSE)), add=TRUE, col="green",lwd=1)
	
	if(method=="Normal"){
		smoothSD<-qnorm(quantileV,mean=kernel.sd,sd=my.sd)
	}
	if(method=="Lognormal"){
		## Fit a log-normal distribution using the MASS package
		gg<-fitdistr(SDvector,"log-Normal")
		curve(dlnorm(x,meanlog=gg$estimate[1],sdlog=gg$estimate[2]),add=TRUE,col="blue")
		smoothSD<-qlnorm(quantileV, meanlog = gg$estimate[1],sdlog=gg$estimate[2])
		if(smoothSD>kernel.sd){
			smoothSD<-kernel.sd
		}
	}
	if(method=="Boxplot"){ #25th quantile
		smoothSD<-v$stats[2] #lower hinge
	}
	if(method=="BoxplotMedian"){
		smoothSD<-v$stats[3] #Median
	}
	if(method=="BoxplotUpper"){ #75th quantile
		smoothSD<-v$stats[4] 
	}
	if(method=="BoxplotFree"){ #75th quantile
		smoothSD<-quantile(SDvector,quantileV)
	}	
	
	if(method=="Kernel"){
		smoothSD<-kernel.sd
	}	
	legend("topright",c("Kernel estimation","Normal estimation","Lognormal estimation"),bty="n",xjust=1,cex=0.7,col=c("red","green","blue"),lty=1,lwd=2)
	axis(side=3,at=smoothSD,labels=paste("smoothSD= ",round(smoothSD,3),sep=""))
	print(paste("smoothSD from ",method," = ",smoothSD,sep=""))
	abline(v=smoothSD,lwd=2,lty=3)
	abline(v=kernel.sd)
	dev.off()
	
		
	### smoothSD determined, extract which clones to use for the subset M vector
	ii<-which(SDvector<=smoothSD)
	ii2<-match(SDrepvector[ii],data$position)
	Mdata<-cbind(data$position[ii2],data$chromosome[ii2],data$startPosition[ii2],data$l2ratio1_2[ii2],data$l10intgmean1_2[ii2])
	colnames(Mdata)<-c("position","chromosome","startPosition","l2ratio1_2","l10intgmean1_2")
	Mdata<-as.data.frame(Mdata)
	Mchromosomes<-sort(unique(Mdata$chromosome))
	
	ReporterNbrSummary[i]<-length(Mdata$l2ratio1_2)
	rm(SDvector,SDrepvector,ii,ii2,ff.sd,v)
	gc();
	##
	
		
	### Do the segmentation per chromosome on Mvector, = Mvalues passing the smoothSD cut
	all.levels<-vector()
	
	if(segMethod=="cghplotter"){
		#cghplotter segmentation on a chromosome arm basis
		nbrClusterCenter<-3  # Definition of nbr cluster centers for segmentation
		init.pt<-3
		max.iter<-100
		pc<-0.02
		for(j in 1:length(Mchromosomes)){
			Mdata.subset<-Mdata[which(Mdata$chromosome==Mchromosomes[j]),]
			
			## Divide into p-arm and q-arm data		
			my.pdata<-Mdata.subset$l2ratio1_2[which(Mdata.subset$startPosition<centromer.limit[j])]
			my.qdata<-Mdata.subset$l2ratio1_2[which(Mdata.subset$startPosition>=centromer.limit[j])]
		
		
			## Segmentation part
			if((length(my.pdata)>0) && (length(my.qdata)>0)){
				c.p<-1
				c.q<-1
				if(length(my.pdata)>nbrClusterCenter){
					c.p<- Compute_kmean_and_levels(my.pdata,noiseconstant,init.pt,max.iter,pc)
				}
				levels.p<-Dynamic_prog(my.pdata,c.p)
				if(length(my.qdata)>nbrClusterCenter){
					c.q<- Compute_kmean_and_levels(my.qdata,noiseconstant,init.pt,max.iter,pc)
				}
				levels.q<-Dynamic_prog(my.qdata,c.q)
			
				print(paste("c.p=",c.p))
				print(paste("c.q=",c.q))
				print(paste("c.p + c.q=",(c.q+c.p)))
		
				## combine levels.p and q to normal levels
				all.levels<-c(all.levels,levels.p$step,levels.q$step)
				print(paste("Done chr ",j) )
				rm(levels.p,levels.q,c.p,c.q)
			}else{
				if(length(my.qdata)>0){
					#data exists on q-arm
					c.q<- Compute_kmean_and_levels(my.qdata,noiseconstant,init.pt,max.iter,pc)
					levels.q<-Dynamic_prog(my.qdata,c.q)
					all.levels<-c(all.levels,levels.q$step)
					print(paste("c.q=",c.q))
					rm(c.q,levels.q)
					print(paste("Done chr ",j) )
				}
				if(length(my.pdata)>0){
					#data exists on q-arm
					c.p<- Compute_kmean_and_levels(my.pdata,noiseconstant,init.pt,max.iter,pc)
					levels.p<-Dynamic_prog(my.pdata,c.p)
					all.levels<-c(all.levels,levels.p$step)
					print(paste("c.p=",c.p))
					rm(c.p,levels.p)
					print(paste("Done chr ",j) )
				}
			}			
		}
		rm(nbrClusterCenter)
	}#end cghplotter segmentation
	gc();
	
	all.levels<-round(all.levels,3) #safety
	#nbrClusters<-3 # requires definition by user now!!
	my.col<-c("yellow","red","green","orange","blue")
	
	
	my.centers<-vector()
	for(q in 1:50){
		dd.rand<-kmeans(all.levels,nbrClusters,iter.max=500) #cluster using the selected number	
		my.centers<-c(my.centers,dd.rand$center)
	}
	my.centers.hist<-hist(my.centers,plot=FALSE) #breaks?
	hh<-rbind(my.centers.hist$counts,my.centers.hist$mids)
	uu<-order(hh[1,],decreasing=TRUE)
	hh<-hh[,uu]
	my.new.centers<-sort(hh[2,1:nbrClusters]) #3 centers!
	print(my.new.centers)
	rm(dd.rand,hh,uu)
	gc();
	
	if(mergeClusterMethod=="z"){
		## Run a kmean clustering assumin k=3 clusters
		dd<-kmeans(all.levels,my.new.centers,iter.max=3000) #cluster using the selected number	
		dd$center
		m1<-mean(Mdata$l2ratio1_2[which(dd$cluster==1)])
		s1<-sd(Mdata$l2ratio1_2[which(dd$cluster==1)])
		m2<-mean(Mdata$l2ratio1_2[which(dd$cluster==2)])
		s2<-sd(Mdata$l2ratio1_2[which(dd$cluster==2)])
		m3<-mean(Mdata$l2ratio1_2[which(dd$cluster==3)])
		s3<-sd(Mdata$l2ratio1_2[which(dd$cluster==3)])
		z1.3<-abs(m1-m3)/sqrt(s1^2+s3^2)
		z1.2<-abs(m1-m2)/sqrt(s1^2+s2^2)
		z2.3<-abs(m2-m3)/sqrt(s2^2+s3^2)
		if(z1.3<=mergeCluster){
			#merge into 1 cluster
			nbrClusters<-1
			my.new.centers<-mean(my.new.centers)
			rm(test1.3)
		}else{
			if(z1.2<=mergeCluster){
				#merge 1 and 2
				nbrClusters<-2
				my.new.centers<-c(mean(my.new.centers[1:2]),my.new.centers[3])
			}else{
				if(z2.3<=mergeCluster){
					#merge 1 and 2
					nbrClusters<-2
					my.new.centers<-c(my.new.centers[1],mean(my.new.centers[2:3]))
				}
			}
		}
	}#end z merge method
	if(mergeClusterMethod=="global"){
		## Check if to centers are to close to one another, if so merge them, and subtract 1 cluster
		#  from the total number (originally 3)
		if(length(my.new.centers)==3){
			if(abs(my.new.centers[1]-my.new.centers[3])<=mergeCluster){
				# the two extreme clusters are very close to each other, cluster 2 between them,  
				# use only one cluster all in all therefore
				nbrClusters<-1
				my.new.centers<-mean(my.new.centers)
			}else{
				if(abs(my.new.centers[1]-my.new.centers[2])<=mergeCluster){
					#cluster 1 and 2 to close
					nbrClusters<-2
					my.new.centers<-c(mean(my.new.centers[1:2]),my.new.centers[3])
				}else{
					if(abs(my.new.centers[2]-my.new.centers[3])<=mergeCluster){
						#cluster 2 and 3 to close
						nbrClusters<-2
						my.new.centers<-c(my.new.centers[1],mean(my.new.centers[2:3]))
					}	
				}
			}
		}#end if nbrClusters = 3
		
		if(length(my.new.centers)==5){
			#strategy:
			#1. check cluster 1 - 5 -> 1 cluster
			#2. check cluster 2 - 4 -> 3 clusters
			#3. check cluster 3 - 4 -> 4 clusters
			#4. check cluster 2 - 3 -> 4 clusters
			if(abs(my.new.centers[1]-my.new.centers[5])<=mergeCluster){
				# the two extreme clusters are very close to each other, cluster 2 between them,  
				# use only one cluster all in all therefore
				nbrClusters<-1
				my.new.centers<-mean(my.new.centers)
			}else{
				if(abs(my.new.centers[2]-my.new.centers[4])<=mergeCluster){
					nbrClusters<-3
					my.new.centers<-c(my.new.centers[1],mean(my.new.centers[2:4]),my.new.centers[5])
				}else{
					if(abs(my.new.centers[3]-my.new.centers[4])<=mergeCluster){
						nbrClusters<-4
						my.new.centers<-c(my.new.centers[1],my.new.centers[2],mean(my.new.centers[3:4]),my.new.centers[5])
					}else{
						if(abs(my.new.centers[2]-my.new.centers[3])<=mergeCluster){
							nbrClusters<-4
							my.new.centers<-c(my.new.centers[1],mean(my.new.centers[2:3]),my.new.centers[4],my.new.centers[5])
						}
					}
				}
				
			}
		}
	}#end method with hard cut M-merge
	if(mergeClusterMethod=="sdcut"){
		#if cluster centers differ by less than the smoothSD value, they will be joined!
		if(length(my.new.centers)==3){
			if(abs(my.new.centers[1]-my.new.centers[3])<=smoothSD){
				# the two extreme clusters are very close to each other, cluster 2 between them,  
				# use only one cluster all in all therefore
				nbrClusters<-1
				my.new.centers<-mean(my.new.centers)
			}else{
				if(abs(my.new.centers[1]-my.new.centers[2])<=smoothSD){
					#cluster 1 and 2 to close
					nbrClusters<-2
					my.new.centers<-c(mean(my.new.centers[1:2]),my.new.centers[3])
				}else{
					if(abs(my.new.centers[2]-my.new.centers[3])<=smoothSD){
						#cluster 2 and 3 to close
						nbrClusters<-2
						my.new.centers<-c(my.new.centers[1],mean(my.new.centers[2:3]))
					}	
				}
			}
		}#end if nbrClusters = 3
		
		if(length(my.new.centers)==5){
			#strategy:
			#1. check cluster 1 - 5 -> 1 cluster
			#2. check cluster 2 - 4 -> 3 clusters
			#3. check cluster 3 - 4 -> 4 clusters
			#4. check cluster 2 - 3 -> 4 clusters
			if(abs(my.new.centers[1]-my.new.centers[5])<=smoothSD){
				# the two extreme clusters are very close to each other, cluster 2 between them,  
				# use only one cluster all in all therefore
				nbrClusters<-1
				my.new.centers<-mean(my.new.centers)
			}else{
				if(abs(my.new.centers[2]-my.new.centers[4])<=smoothSD){
					nbrClusters<-3
					my.new.centers<-c(my.new.centers[1],mean(my.new.centers[2:4]),my.new.centers[5])
				}else{
					if(abs(my.new.centers[3]-my.new.centers[4])<=smoothSD){
						nbrClusters<-4
						my.new.centers<-c(my.new.centers[1],my.new.centers[2],mean(my.new.centers[3:4]),my.new.centers[5])
					}else{
						if(abs(my.new.centers[2]-my.new.centers[3])<=smoothSD){
							nbrClusters<-4
							my.new.centers<-c(my.new.centers[1],mean(my.new.centers[2:3]),my.new.centers[4],my.new.centers[5])
						}
					}
				}
				
			}
		}
	}#end method merge by smoothSD
	
	dd<-list()
	if(nbrClusters==1){
		#create dd
		dd$centers<-my.new.centers
		dd$cluster<-rep(1,length(all.levels))
		dd$size<-length(all.levels)
	}else{
		dd<-kmeans(all.levels,my.new.centers,iter.max=3000) #cluster using the selected number	
		dd$center
	}
	gc();
	
	
	## Plot the distribution of segmented levels
	myfile<-paste(assayName,"_BeforeLowessPopulations.ps",sep="")
	my.plotfiles[2,1]<-myfile
	postscript(myfile)
	op <- par(mfrow = c(2, 1),mar=c(4, 4, 2, 1))
	
	my.segs<-unique(c(min(all.levels,na.rm=TRUE),seq(min(all.levels,na.rm=TRUE),max(all.levels,na.rm=TRUE),by=binSize),max(all.levels,na.rm=TRUE)) )
	my.maxs<-rep(0,nbrClusters)
	for(j in 1:nbrClusters){
		test<-hist(all.levels[dd$cluster==j],plot=FALSE,breaks=my.segs)
		my.maxs[j]<-max(test$counts)
		rm(test)
	}
	
	## Result after Kmean clustering
	jj<-sort(dd$size,index.return=TRUE,decreasing=TRUE)
	for(j in nbrClusters:1){
		if(j==nbrClusters){
			hist(all.levels[which(dd$cluster==jj$ix[j])],breaks=seq(min(all.levels)-0.05,max(all.levels)+0.05,by=0.025),cex.lab=0.9,main="",cex.axis=0.9,ylim=c(0,max(my.maxs)),las=1,xaxt="n",xlab="log2ratio",xlim=range(all.levels),col=my.col[j])
			title(main="Histogram of all segments M-value, coloured by cluster belonging",cex.main=0.9)
			axis(side=1,at=seq(round(min(all.levels)),round(max(all.levels)),by=0.25),labels=seq(round(min(all.levels)),round(max(all.levels)),by=0.25),cex.axis=0.9)
			abline(v=dd$center[jj$ix[j]],lwd=1)
		}else{
			hist(all.levels[dd$cluster==jj$ix[j]],breaks=seq(min(all.levels)-0.05,max(all.levels)+0.05,by=0.025),cex.lab=0.9,add=TRUE,col=my.col[j])
			abline(v=dd$center[jj$ix[j]],lwd=1)
		}
	}
	rm(my.maxs,my.segs)
	#dev.off()
		
	
	### Extract the data for each cluster
	lz<-list()
	
	for(k in 1:nbrClusters){
		my.segs<-unique(all.levels[dd$cluster==k]) #if not all.levels is unique, this is required!
		# get appropiate data from all.levels
		clusterSeg<-vector()
		clusterPos<-vector()
		clusterA<-vector()
		clusterM<-vector()
		for(j in 1:length(my.segs)){
			ii<-which(all.levels==my.segs[j])
			clusterSeg<-c(clusterSeg,all.levels[ii])
			clusterPos<-c(clusterPos,Mdata$position[ii])
			clusterA<-c(clusterA,Mdata$l10intgmean1_2[ii])
			clusterM<-c(clusterM,Mdata$l2ratio1_2[ii])
		}
		lz[[length(lz)+1]]<-cbind(clusterSeg,clusterPos,clusterA,clusterM)
		rm(clusterSeg,clusterPos,clusterA,clusterM,my.segs,ii)
	}
		
	## continue plotting before lowess populations
	my.pop.m.b<-matrix(nrow=nbrClusters,ncol=5,0)
	my.pop.m.b.2<-matrix(nrow=nbrClusters,ncol=5,0)
		
	this.col<-my.col
	for(h in nbrClusters:1){
		clusterData<-lz[[jj$ix[h]]]
		vh<-boxplot.stats(clusterData[,4])
		my.pop.m.b[h,]<-vh$stats
		rm(clusterData,vh)
		this.col[h]<-my.col[h]
	}
	boxplot(as.data.frame(t(my.pop.m.b)),las=1,main="Populations sorted by size",cex.main=0.7,names=paste("Pop_",jj$ix,sep=""),col=this.col,ylab="M")
	rm(my.pop.m.b,this.col,jj)
	dev.off() #before lowess populations
	gc();
	
		
	### Calculate the global median corrected M-values
	global.m<-median(data$l2ratio1_2,na.rm=TRUE)
	global.m.corr<- 0-global.m
	
		
	### Calculate the standard lowess function, correctionvalues and the new corrected value ###
	standardLowess<-lowess(data$l10intgmean1_2,data$l2ratio1_2,f=lowessSpan,iter = lowessIter,delta=lowessDelta) #standard lowess using uncorrected data!
	ii<-sort(data$l10intgmean1_2,index.return=TRUE)
	ss<-cbind(standardLowess$x,data$l10intgmean1_2[ii$ix],(data$l10intgmean1_2[ii$ix]-standardLowess$x))
	ss.f<-which(ss[,3] !=0)
	print(paste("number of wrong backmaps from A matching for lowess = ", length(ss.f)))
							# sorted as data$l2ratio1_2
	rm(ii,ss,ss.f)
	gc();
	###
	
		
	### Correct for curvature if poplowess selected
	if(normMethod=="lowess"){
		my.round<-3 #number of digits to round to
		# cluster order in dd is the same as for lz
		clusterData<-lz[[which.max(dd$size)]]
		lowessL<-lowess(clusterData[,3],clusterData[,4],f=lowessSpan,iter=lowessIter,delta=lowessDelta)
		lowessLpos<-clusterData[,2] #the actual clones used
		rm(clusterData)
		gc();
		
		ggg<-round(lowessL$x,my.round)
		a.range<-seq(round(min(data$l10intgmean1_2),my.round),round(max(data$l10intgmean1_2),my.round),by=1/(10^my.round))
		a.range<-round(a.range,my.round)
		m.corr<-rep(NA,length(a.range))
		m.index<-seq(1,length(a.range))
		m.index.logical<-rep(FALSE,length(a.range))
		for(k in 1:length(a.range)){
			m<-which(ggg==a.range[k]) #max one hit because unique
			if(length(m)>0){
				m.corr[k]<-mean(lowessL$y[m]) #if multiple values that differ!!
				m.index.logical[k]<-TRUE
				#print(m.corr[k])
			}
		}
		## Now fill up all blank slots (NA) in m.corr
		for(k in 1:length(m.corr)){
			if(is.na(m.corr[k])){
				# index containing NA
				# look before k
				before<-m.index.logical[-seq(k,length(m.corr))]
				beforeD<-m.corr[-seq(k,length(m.corr))]
				if(length(which(m.index.logical[1:k]))>0){
					#true index found = lowess value
					my.lower<-beforeD[max(which(before))]
				}else{
					my.lower<-NA
				}
				after<-m.index.logical[-seq(1,k)]
				afterD<-m.corr[-seq(1,k)]
				if(length(which(after))> 0){
					#true index found = lowess value
					my.upper<-afterD[min(which(after))]
				}else{
					my.upper<-NA
				}
				m.corr[k]<-mean(c(my.lower,my.upper),na.rm=TRUE)
				m.index.logical[k]<-TRUE
				#print(paste("index ",k," is a NA, replaced by value: ",m.corr[k]))
			}
		}
		##
		
		print("finished the extended correction line")
		gc();
		
		m.corr<-0-m.corr #the correction value to add to each clone, depending on its position on the A-axis
		M.zlowess<-data$l2ratio1_2  #this will be the z-lowess corrected data
		
		
		#### Correct the M-values with the z-lowess correction
		# 1. Correct clusters
		jj<-sort(dd$size,index.return=TRUE,decreasing=TRUE)
		for(h in nbrClusters:1){
			clusterData<-lz[[jj$ix[h]]]
			gg<-round(clusterData[,3],my.round)
			for(k in 1:length(a.range)){
				ii<-which(gg==a.range[k])
				clusterData[ii,4]<-clusterData[ii,4]+m.corr[k]
			}
			lz[[jj$ix[h]]]<-clusterData #reassign
		}
		
		# 2. Correct all data
		fff<-round(data$l10intgmean1_2,my.round)
		for(k in 1:length(a.range)){ #foreach correction point
			ii<-which(fff==a.range[k])
			M.zlowess[ii]<-data$l2ratio1_2[ii]+m.corr[k]			
		}
		
					
		#do an MAplot showing the populations and correction line
		A.sort<-a.range[seq(1,length(m.corr),length.out=200)]
		m.corr.sort<-0-m.corr[seq(1,length(m.corr),length.out=200)]
			
		col.f<-rep("black",length(data$position))
		for(h in nbrClusters:1){
			clusterData<-lz[[jj$ix[h]]] #matrix, cols = segmentvalues, positions (identifier), A, M
			ii<-match(clusterData[,2],data$position)
			col.f[ii]<-my.col[h]
		}
		myfile<-paste(assayName,"_MA.ps",sep="")
		my.plotfiles[3,1]<-myfile
		postscript(myfile)
		## M-A unnormalised data with populations
		plot(data$l10intgmean1_2,data$l2ratio1_2,xlab="A",col=col.f,xaxt="n",yaxt="n",ylab="M",las=1,cex=0.25)
		title(main="M vs A raw data, subpops coloured before any correction")
		axis(side=2,at=seq(round(min(data$l2ratio1_2)),round(max(data$l2ratio1_2)),by=0.5),las=1,labels=seq(round(min(data$l2ratio1_2)),round(max(data$l2ratio1_2)),by=0.5),cex.axis=0.7)
		axis(side=1,at=seq(round(min(data$l10intgmean1_2)),round(max(data$l10intgmean1_2)),by=0.25),labels=seq(round(min(data$l10intgmean1_2)),round(max(data$l10intgmean1_2)),by=0.25),cex.axis=0.7)
		lines(A.sort,m.corr.sort,lwd=1)
		ii.s<-seq(1,length(data$l2ratio1_2),length.out=200) #subset sequence
		lines(standardLowess$x[ii.s],standardLowess$y[ii.s],lwd=2,col="green") # adding the standard lowess line!
		abline(h=global.m,col="yellow",lwd=2)
		legend("bottomleft",c("Lowess","popLowess","globalMedian"),bty="n",xjust=1,cex=0.7,col=c("green","red","yellow"),lty=1,lwd=2)
		dev.off()
		rm(col.f,ii.s)
	}else{
		# global median adjustment to the largest population!
		clusterData<-lz[[which.max(dd$size)]]
		pop.corr<-0-median(clusterData[,4],na.rm=TRUE)
		rm(clusterData)
		
		# correct all data
		M.zlowess<-data$l2ratio1_2+pop.corr
		
		# correct clusters
		#jj<-sort(dd$size,index.return=TRUE,decreasing=TRUE)
		#for(h in nbrClusters:1){
		#	clusterData<-lz[[jj$ix[h]]]
		#	clusterData[,4]<-clusterData[,4]+pop.corr
		#	lz[[jj$ix[h]]]<-clusterData #reassign
		#	rm(clusterData)
		#}
		#rm(jj)
		gc();
	}
	gc();


	### PEAK selection
		
	# Methodology:
	# 1. If number of clusters = 3 select the middle cluster to center on.
	# 2. If number of clusters < 3 select the largest cluster to center on.
	#
	#
	
	corrM<-0 
	clusterMedian<-rep(NA,nbrClusters)
		
	for(j in 1:length(lz)){
		#remember here that lz already contains normalised values centred to the largest population!
		clusterData<-lz[[j]] #matrix, cols = segmentvalues, positions (identifier), A, M
		clusterMedian[j]<-median(clusterData[,4])
		rm(clusterData)
	}
	gc();
	
		
	### Check for constraints
	if(length(which(constraintA==assaysToRow$Assay[i]))>0){
		#Constraint HIT! Only one by default. Get constraint value.
	
		cV<-constraintV[which(constraintA==assaysToRow$Assay[i])]
		a1<- abs(clusterMedian-cV)
		corrM<-clusterMedian[which.min(a1)]
		rm(a1,cV)
	}
	###

	### Final centralization adjustment from above
	corrM<-0-corrM #peak adjustment factor
	M.zlowess<-M.zlowess+corrM #adjust all data
	CorrectionSummary[i,1]<-mean(data$l2ratio1_2)
	CorrectionSummary[i,2]<-mean(M.zlowess)
		
	#adjust clusters
	jj<-sort(dd$size,index.return=TRUE,decreasing=TRUE)
	
	#for(h in nbrClusters:1){
	#	clusterData<-lz[[jj$ix[h]]]
	#	clusterData[,4]<-clusterData[,4]+corrM
	#	lz[[jj$ix[h]]]<-clusterData #reassign
	#	rm(clusterData)
	#}
	gc();
	###
	
	
	## Final genome plot with adjustments for zlowess + peak selection
	myfile<-paste(assayName,"_wholeGenomeFinished.ps",sep="")
	my.plotfiles[8,1]<-myfile
	postscript(myfile)
	
	col.f<-rep("black",length(data$position))
	for(h in nbrClusters:1){
		clusterData<-lz[[jj$ix[h]]] #matrix, cols = segmentvalues, positions (identifier), A, M
		ii<-match(clusterData[,2],data$position)
		col.f[ii]<-my.col[h]
	}
	plot(M.zlowess,cex=0.25,las=1,ylab="M",cex.axis=0.7,cex.lab=0.7,yaxt="n",col=col.f,xaxt="n",xlab="Chromosome")
	title(main=paste("Whole genome after zlowess + peak with sub pop for ",assayName),cex.main=0.8)
	abline(h=c(-0.5,0.5,1,-1,-1.5,1.5,0.3,-0.3),lty=3)
	abline(h=0)
	axis(side=2,at=seq(round(min(M.zlowess)),round(max(M.zlowess)),by=0.5),las=1,labels=seq(round(min(M.zlowess)),round(max(M.zlowess)),by=0.5),cex.axis=0.7)
	# Plot the vertical lines at chromosome boundries and create the values for the x-axis
	bound.index<-c(1,chr.MaxPosOrder) #add a first line
	abline(v=bound.index, lty = 1,col="black")
			
	# Arrange the x-axis with the chromosome names
	at.tics1<-matrix(ncol=length(chromosomes)/2,nrow=1,NA)
	at.tics2<-matrix(ncol=length(chromosomes)/2,nrow=1,NA)
	as.lab1<-matrix(ncol=length(chromosomes)/2,nrow=1,"")
	as.lab2<-matrix(ncol=length(chromosomes)/2,nrow=1,"")
	for(b in 1:length(chromosomes)){
		if(b%%2){
			at.tics1[b]<-mean(c(bound.index[b],bound.index[b+1]))
			as.lab1[b]<-chromosomes[b]
		}else{
			at.tics2[b]<-mean(c(bound.index[b],bound.index[b+1]))
			as.lab2[b]<-chromosomes[b]
		}
	}
	axis(side=1,labels=as.lab1,at=at.tics1,tick=TRUE,cex.axis=0.7,las=1,font=2,lty=0) #bottom sample name axis
	#axis(side=3,labels=as.lab2,at=at.tics2,tick=TRUE,cex.axis=0.7,las=1,font=2,lty=0) #bottom sample name axis
	rm(at.tics1,at.tics2,as.lab1,as.lab2)
	
	dev.off()	
	
	
	### Adjust the real data (all)
		#keep A fixed = data$l10intgmean1_2
		#log10(sqrt(data$intensity1*data$intensity2))
		# r.vec = data$intensity1/data$intensity2
		# ch2= (10^A) /sqrt(r)
		# ch1= r*ch2
	r.vec<-2^(M.zlowess) #new ratio
	
	data$intensity2<-(10^data$l10intgmean1_2)/sqrt(r.vec)
	data$intensity1<-r.vec*data$intensity2
	
	#### Create the cghanalysed output file, used to import data back into BASE
	# output file is printed as 
	# [position]	[reporter]	[intensity1]	[intensity2]
	# Write out each assay and chromosome individually!
	myfile<-paste("Analysed_",assaysToRow$FileName[i],sep="")  #file format name is e.g. 'Analysed_Sample1.txt' 
	write.table(cbind(data$position,data$reporter,data$intensity1,data$intensity2), file = myfile, quote = FALSE,append=FALSE, sep = "\t", row.names = FALSE,col.names = FALSE)
	####
	gc();
	
	
	## Write the plot file names to specific file for later directory moving in PERL script
	write.table(my.plotfiles, file = assaysToRow$PlotFileName[i], quote = FALSE,append=FALSE, sep = "\t", row.names = FALSE,col.names = FALSE)
	rm(my.plotfiles,r.vec,data,bound.index,M.zlowess,jj,lz,corrM,PosOrder)
}# end for specific assay

#print the correction factor summary
CorrectionSummary<-cbind(assaysToRow$Assay,CorrectionSummary)
colnames(CorrectionSummary)<-c("Assay","preMeanM","postMeanM_corrFactor")
write.table(CorrectionSummary, file = "CorrectionSummary.txt", quote = FALSE,append=FALSE, sep = "\t", row.names = FALSE,col.names = TRUE)


## Cross assay comparison plots
myfile<-"Cross_Assay_NbrReportersUsed.ps"
postscript(myfile)
par(omd=c(0.025,1,0,1))
barplot(ReporterNbrSummary,col="grey76",names.arg=assaysToRow$Assay,cex.main=0.9,cex.lab=0.9,main="Number reporters used for estimation",las=2,cex.axis=0.7,ylab="# reporters")
par(op)
dev.off()