popLowess<-function(reporter,chr,start,stop,int.matrix,sdCutMethod="median",alpha=0.05,nperm=1000,normMethod="lowess",windowType="percent",windowSize=0.005,lowessSpan=0.33,lowessDelta=0.1,lowessIter=4,mergeClusterM=0,excludeChr=c(24),assayNames=c(1)){
	### Parameters ###
	#reporterId vector
	#chromosome vector
	#startPosition vector
	#endPosition vector
	#int.matrix to normalize
	#sdCutMethod (median) implemented
	#normMethod  (lowess) or "global"
	#windowType (percent or probes)
	#windowSize (half nbr probes or half % of total nbr probes on chromosome)
	#lowessSpan
	#lowessDelta
	#lowessIter
	#mergeClusterM  (a value in M. If two populations centers differ by less, they are joined.)
	#excludeChr (excludes chromosomes from the calculations but normalizes them in the end)
	#alpha  : alpha for DNAcopy
	#nperm  : nperm for DNAcopy
	## Not implemented: constraints!
	binSize<-0.05
	nbrClusters<-3
	smoothSD<-0 #assume
	nbrAssays<-length(int.matrix[1,])/2
	posOrder<-seq(1,length(int.matrix[,1])) #position order
	uu<-2
	while(length(assayNames)<nbrAssays){
		assayNames<-c(assayNames,uu)
		uu<-uu+1
	}
	rm(uu)
	mod.int.matrix<-int.matrix #bears the normalised intensities
	
	
	## For each assay normalize and return
	loop.control<-seq(1,nbrAssays*2,by=2)
	for(i in 1:length(loop.control)){
		print(assayNames[i])
		int1<-int.matrix[,loop.control[i]]
		int2<-int.matrix[,loop.control[i]+1]
		
		M<-log2(int1/int2)
		l10intgmean1_2<-log10(sqrt(int1*int2))	#A, log10(sqrt(Ch1*Ch2))
		
		## Define chromosomes for this sample
		chromosomes<-sort(unique(chr))
		chr.MaxPosOrder<-rep(0,length(chromosomes))
		for(j in 1:length(chromosomes)){
			chr.MaxPosOrder[j]<-max(which(data$chromosome==chromosomes[j]),na.rm=TRUE)
		}
		##

		
		## Exclude options (exclude chromosomes from calculation, however do not filter them out)
		ii<-match(excludeChr,chromosomes)
		chromosomesC<-vector()
		if(length(ii[which(!is.na(ii))])>0){
			chromosomesC<-chromosomes[-c(ii)]	
		}else{
			chromosomesC<-chromosomes
		}
		##
				
		
		## Do the SD smoothing
		SDvector<-vector() #collects the window SD:s
		SDrepvector<-vector() #to collect positions
		uu<-which(is.na(M))
		M.window<-M[-uu] #remove missing values
		posOrder.window<-posOrder[-uu]
		chr.window<-chr[-uu]
		rm(uu)
		for(c in 1:length(chromosomesC)){
			workingChrom<-chromosomesC[c]
			data.subset<-M.window[which(chr.window==workingChrom)]
			position.subset<-posOrder.window[which(chr.window==workingChrom)]
			r<-1:length(data.subset)
			
			##assuming smooth is a percentage of clones residing on a chromosome
			smooth.size<-round(windowSize*length(data.subset))
			
			if(windowType=="probes"){
				#WINDOW defined by probes, i.e. smooth = integer!
				smooth.size<-windowSize
			}		
			
			r.start<-r-smooth.size
			r.end<-r+smooth.size
			
			#trimming the vectors
			ii<-seq(1+smooth.size,length(r)-smooth.size)
			r.run<-r[ii]
			r.start<-r.start[ii]
			r.end<-r.end[ii]
			
			localV<-rep(0,length(r.run))
			localR<-rep(0,length(r.run))
			for(u in 1:length(r.run)){
				localV[u]<-sd(data.subset[r.start[u]:r.end[u]],na.rm=TRUE)
				localR[u]<-position.subset[u]
			}
			SDvector<-c(SDvector,localV)
			SDrepvector<-c(SDrepvector,localR)
			rm(localV,localR,r.start,r.end,r.run,ii,r,data.subset,position.subset)
		}#end smoothing
		rm(M.window,posOrder.window,chr.window)
		gc();
		##
		
		## clear any NA:s introduced..
		na.indexes<-which(is.na(SDvector))
		if(length(na.indexes)>0){
			SDvector<-SDvector[-na.indexes]
			SDrepvector<-SDrepvector[-na.indexes]
		}
		rm(na.indexes)	
		gc();
		##
	
		
		## Define the smoothSD cut off. Median only implemented
		smoothSD<-median(SDvector)
		##
		
		
		## Make a SD histogram plot 
		myfile<-paste(assayNames[i],"_SDplot.ps",sep="")
		postscript(myfile)
		v<-boxplot.stats(SDvector,coef = 1.5)
		ff.sd<-density(SDvector, kernel="gaussian", bw=0.01) #estimate kernel function
		kernel.sd<-ff.sd$x[which.max(ff.sd$y)]
		u.d<-ff.sd$y[which.max(ff.sd$y)]
		u.1sd<-0.607*u.d #the height at 1 sd for a normal distribution with mean u. 
		lower<-ff.sd$y[which(ff.sd$x<kernel.sd)]
		upper<-ff.sd$y[which(ff.sd$x>kernel.sd)]
		lowD<-max(lower[which(lower<=u.1sd)])
		highD<-max(upper[which(upper<=u.1sd)])
		lowSD<-ff.sd$x[which(ff.sd$y==lowD)]
		highSD<-ff.sd$x[which(ff.sd$y==highD)]
		my.sd<-mean(c(kernel.sd-lowSD,highSD-kernel.sd))
		rm(u.d,u.1sd,lower,upper,lowD,highD,lowSD,highSD)
			
		my.segBreaks<-round( ( max(SDvector,na.rm=TRUE)-min(SDvector,na.rm=TRUE))/(1*binSize))
		vvv<-hist(SDvector,breaks=my.segBreaks,plot=FALSE)
		my.ylim<-c(0,max(vvv$density)+3)
		rm(vvv)
		op <- par(omd=c(0,1,0,0.93)) #increase the outer margins slightly		
		hist(SDvector,breaks=my.segBreaks,freq=FALSE,ylim=my.ylim,las=1,xaxt="n",cex.main=0.8,main="",xlab="sd")
		title(main="Histogram of SDvector",outer=TRUE,cex.main=0.9)
		axis(side=1,at=seq(0,round(max(SDvector,na.rm=TRUE)),by=0.1),labels=seq(0,round(max(SDvector,na.rm=TRUE)),by=0.1),cex.axis=0.7)
		lines(ff.sd,col="red")
		legend("topright",c("Kernel estimation"),bty="n",xjust=1,cex=1,col=c("red"),lty=1,lwd=2)
		axis(side=3,at=smoothSD,labels=paste("smoothSD= ",round(smoothSD,3),sep=""))
		abline(v=smoothSD,lwd=2,lty=3)
		abline(v=kernel.sd)
		dev.off()
		##
	
		
		### smoothSD determined, extract which clones to use for the subset M vector
		ii<-which(SDvector<=smoothSD)
		ii2<-match(SDrepvector[ii],posOrder)
		Mdata<-cbind(posOrder[ii2],chr[ii2],start[ii2],M[ii2],l10intgmean1_2[ii2])
		colnames(Mdata)<-c("position","chromosome","startPosition","l2ratio1_2","l10intgmean1_2")
		Mdata<-as.data.frame(Mdata)
		Mchromosomes<-sort(unique(Mdata$chromosome))
		rm(SDvector,SDrepvector,ii,ii2,ff.sd,v)
		gc();
	
	
		### Do the segmentation (CBS) per chromosome on Mvector, = Mvalues passing the smoothSD cut
		all.levels<-vector()
		segmented<- segment(CNA(Mdata$l2ratio1_2, Mdata$chromosome, Mdata$startPosition, data.type=c("logratio"),sampleid=assayNames[i]),alpha = alpha, nperm=nperm,verbose=2)
		## Assign each clone its segment value
		for(j in 1:length(Mchromosomes)){
			assay.segment.subset.object <- subset.DNAcopy(segmented,chromlist=Mchromosomes[j])
			i1<-match(assay.segment.subset.object$output$loc.start, assay.segment.subset.object$data$maploc)
			i2<-match(assay.segment.subset.object$output$loc.end, assay.segment.subset.object$data$maploc)
			for(k in 1:length(i1)){
				all.levels<-c(all.levels,rep(assay.segment.subset.object$output$seg.mean[k],each=assay.segment.subset.object$output$num.mark[k]))
			}
			rm(assay.segment.subset.object,i1,i2)
		}
		rm(segmented)
		gc();


		all.levels<-round(all.levels,3) #safety
		my.col<-c("yellow","red","green","orange","blue")
	
		my.centers<-vector()
		for(q in 1:50){
			dd.rand<-kmeans(all.levels,nbrClusters,iter.max=500) #cluster using the selected number	
			my.centers<-c(my.centers,dd.rand$center)
		}
		my.centers.hist<-hist(my.centers,plot=FALSE) #breaks?
		hh<-rbind(my.centers.hist$counts,my.centers.hist$mids)
		uu<-order(hh[1,],decreasing=TRUE)
		hh<-hh[,uu]
		my.new.centers<-sort(hh[2,1:nbrClusters]) #3 centers!
		print(my.new.centers)
		rm(dd.rand,hh,uu)
		gc();
		gc();


		## Check if to centers (3) are to close to one another, 
		#  if so merge them, and subtract 1 cluster from the total number (originally 3)
		if(length(my.new.centers)==3){
			if(abs(my.new.centers[1]-my.new.centers[3])<=mergeClusterM){
				# the two extreme clusters are very close to each other, cluster 2 between them,  
				# use only one cluster all in all therefore
				nbrClusters<-1
				my.new.centers<-mean(my.new.centers)
			}else{
				if(abs(my.new.centers[1]-my.new.centers[2])<=mergeClusterM){
					#cluster 1 and 2 to close
					nbrClusters<-2
					my.new.centers<-c(mean(my.new.centers[1:2]),my.new.centers[3])
				}else{
					if(abs(my.new.centers[2]-my.new.centers[3])<=mergeClusterM){
						#cluster 2 and 3 to close
						nbrClusters<-2
						my.new.centers<-c(my.new.centers[1],mean(my.new.centers[2:3]))
					}	
				}
			}
		}#end if nbrClusters = 3
		
		dd<-list()
		if(nbrClusters==1){
			#create dd
			dd$centers<-my.new.centers
			dd$cluster<-rep(1,length(all.levels))
			dd$size<-length(all.levels)
		}else{
			dd<-kmeans(all.levels,my.new.centers,iter.max=3000) #cluster using the selected number	
			dd$center
		}
		gc();	
		
		
		## Plot the distribution of segmented levels
		myfile<-paste(assayNames[i],"_BeforeLowessPopulations.ps",sep="")
		postscript(myfile)
		op <- par(mfrow = c(2, 1),mar=c(4, 4, 2, 1))
		my.segs<-unique(c(min(all.levels,na.rm=TRUE),seq(min(all.levels,na.rm=TRUE),max(all.levels,na.rm=TRUE),by=binSize),max(all.levels,na.rm=TRUE)) )
		my.maxs<-rep(0,nbrClusters)
		for(j in 1:nbrClusters){
			test<-hist(all.levels[dd$cluster==j],plot=FALSE,breaks=my.segs)
			my.maxs[j]<-max(test$counts)
			rm(test)
		}
		
		
		## Result after Kmean clustering
		jj<-sort(dd$size,index.return=TRUE,decreasing=TRUE)
		for(j in nbrClusters:1){
			if(j==nbrClusters){
				hist(all.levels[which(dd$cluster==jj$ix[j])],breaks=seq(min(all.levels)-0.05,max(all.levels)+0.05,by=0.025),cex.lab=0.9,main="",cex.axis=0.9,ylim=c(0,max(my.maxs)),las=1,xaxt="n",xlab="log2ratio",xlim=range(all.levels),col=my.col[j])
				title(main="Histogram of all segments M-value, coloured by cluster belonging",cex.main=0.9)
				axis(side=1,at=seq(round(min(all.levels),2),round(max(all.levels),2),by=0.1),labels=seq(round(min(all.levels),2),round(max(all.levels),2),by=0.1),cex.axis=0.9)
				abline(v=dd$center[jj$ix[j]],lwd=1)
			}else{
				hist(all.levels[dd$cluster==jj$ix[j]],breaks=seq(min(all.levels)-0.05,max(all.levels)+0.05,by=0.025),cex.lab=0.9,add=TRUE,col=my.col[j])
				abline(v=dd$center[jj$ix[j]],lwd=1)
			}
		}
		rm(my.maxs,my.segs)
		gc();
		
		
		### Extract the data for each cluster
		lz<-list()
			
		for(k in 1:nbrClusters){
			my.segs<-unique(all.levels[dd$cluster==k]) #if not all.levels is unique, this is required!
			# get appropiate data from all.levels
			clusterSeg<-vector()
			clusterPos<-vector()
			clusterA<-vector()
			clusterM<-vector()
			for(j in 1:length(my.segs)){
				ii<-which(all.levels==my.segs[j])
				clusterSeg<-c(clusterSeg,all.levels[ii])
				clusterPos<-c(clusterPos,Mdata$position[ii])
				clusterA<-c(clusterA,Mdata$l10intgmean1_2[ii])
				clusterM<-c(clusterM,Mdata$l2ratio1_2[ii])
				uu<-which(is.na(clusterM))
			}
			lz[[length(lz)+1]]<-cbind(clusterSeg,clusterPos,clusterA,clusterM)
			rm(clusterSeg,clusterPos,clusterA,clusterM,my.segs,ii)
		}
				
		## continue plotting before lowess populations
		my.pop.m.b<-matrix(nrow=nbrClusters,ncol=5,0)
		my.pop.m.b.2<-matrix(nrow=nbrClusters,ncol=5,0)
		this.col<-my.col
		for(h in nbrClusters:1){
			clusterData<-lz[[jj$ix[h]]]
			vh<-boxplot.stats(clusterData[,4])
			my.pop.m.b[h,]<-vh$stats
			rm(clusterData,vh)
			this.col[h]<-my.col[h]
		}
		boxplot(as.data.frame(t(my.pop.m.b)),las=1,main="Populations sorted by size",cex.main=0.7,names=paste("ClusterPop_",jj$ix,sep=""),col=this.col,ylab="M")
		rm(my.pop.m.b,this.col,jj)
		
		dev.off() #before lowess populations
		gc();
		
		
		### Calculate the global median corrected M-values
		global.m<-median(M,na.rm=TRUE)
		global.m.corr<- 0-global.m
		###	
			
			
		### Calculate the standard lowess function, correctionvalues and the new corrected value ###
		M.lowess<-M[-which(is.na(M))] #temp remove NA
		A.lowess<-l10intgmean1_2[-which(is.na(M))] #temp remove NA
		standardLowess<-lowess(A.lowess,M.lowess,f=lowessSpan,iter = lowessIter,delta=lowessDelta) #standard lowess using uncorrected data!
		rm(M.lowess,A.lowess)
		gc();
		###
	
				
		### Correct for curvature if poplowess selected
		if(normMethod=="lowess"){
			my.round<-3 #number of digits to round to
			# cluster order in dd is the same as for lz
			clusterData<-lz[[which.max(dd$size)]]
			lowessL<-lowess(clusterData[,3],clusterData[,4],f=lowessSpan,iter=lowessIter,delta=lowessDelta)
			lowessLpos<-clusterData[,2] #the actual clones used
			rm(clusterData)
			gc();
			print("Done lowess on largest subpop")
			
			ggg<-round(lowessL$x,my.round)
			a.range<-seq(round(min(l10intgmean1_2,na.rm=TRUE),my.round),round(max(l10intgmean1_2,na.rm=TRUE),my.round),by=1/(10^my.round))
			a.range<-round(a.range,my.round)
			m.corr<-rep(NA,length(a.range))
			m.index<-seq(1,length(a.range))
			m.index.logical<-rep(FALSE,length(a.range))
			for(k in 1:length(a.range)){
				m<-which(ggg==a.range[k]) #max one hit because unique
				if(length(m)>0){
					m.corr[k]<-mean(lowessL$y[m]) #if multiple values that differ!!
					m.index.logical[k]<-TRUE
				}
			}
			## Now fill up all blank slots (NA) in m.corr
			for(k in 1:length(m.corr)){
				if(is.na(m.corr[k])){
					# index containing NA
					# look before k
					before<-m.index.logical[-seq(k,length(m.corr))]
					beforeD<-m.corr[-seq(k,length(m.corr))]
					if(length(which(m.index.logical[1:k]))>0){
						#true index found = lowess value
						my.lower<-beforeD[max(which(before))]
					}else{
						my.lower<-NA
					}
					after<-m.index.logical[-seq(1,k)]
					afterD<-m.corr[-seq(1,k)]
					if(length(which(after))> 0){
						#true index found = lowess value
						my.upper<-afterD[min(which(after))]
					}else{
						my.upper<-NA
					}
					m.corr[k]<-mean(c(my.lower,my.upper),na.rm=TRUE)
					m.index.logical[k]<-TRUE
				}
			}
			##	
			print("finished the extended correction line")
			rm(m.index.logical,m.index,ggg)
			gc();
		
			m.corr<-0-m.corr #the correction value to add to each clone, depending on its position on the A-axis
			M.popLowess<-M  #this will be the z-lowess corrected data
		
		
			#### Correct the M-values with the poplowess correction
			# 1. Correct clusters
			jj<-sort(dd$size,index.return=TRUE,decreasing=TRUE)
			for(h in nbrClusters:1){
				clusterData<-lz[[jj$ix[h]]]
				gg<-round(clusterData[,3],my.round)
				for(k in 1:length(a.range)){
					ii<-which(gg==a.range[k])
					clusterData[ii,4]<-clusterData[ii,4]+m.corr[k]
				}
				lz[[jj$ix[h]]]<-clusterData #reassign
			}
		
			# 2. Correct all data
			fff<-round(l10intgmean1_2,my.round)
			for(k in 1:length(a.range)){ #foreach correction point
				ii<-which(fff==a.range[k])
				M.popLowess[ii]<-M[ii]+m.corr[k]			
			}
		
					
			#do an MAplot showing the populations and correction line
			A.sort<-a.range[seq(1,length(m.corr),length.out=200)]
			m.corr.sort<-0-m.corr[seq(1,length(m.corr),length.out=200)]
			
			col.f<-rep("black",length(posOrder))
			for(h in nbrClusters:1){
				clusterData<-lz[[jj$ix[h]]] #matrix, cols = segmentvalues, positions (identifier), A, M
				ii<-match(clusterData[,2],data$position)
				col.f[ii]<-my.col[h]
			}
			myfile<-paste(assayNames[i],"_MA.ps",sep="")
			postscript(myfile)
			## M-A unnormalised data with populations
			plot(l10intgmean1_2,M,xlab="A",col=col.f,xaxt="n",yaxt="n",ylab="M",las=1,cex=0.25)
			title(main="M vs A raw data, subpops coloured before any correction")
			axis(side=2,at=seq(round(min(M,na.rm=TRUE)),round(max(M,na.rm=TRUE)),by=0.5),las=1,labels=seq(round(min(M,na.rm=TRUE)),round(max(M,na.rm=TRUE)),by=0.5),cex.axis=0.7)
			axis(side=1,at=seq(round(min(l10intgmean1_2,na.rm=TRUE)),round(max(l10intgmean1_2,na.rm=TRUE)),by=0.25),labels=seq(round(min(l10intgmean1_2,na.rm=TRUE)),round(max(l10intgmean1_2,na.rm=TRUE)),by=0.25),cex.axis=0.7)
			lines(A.sort,m.corr.sort,lwd=1,col="red")
			ii.s<-seq(1,length(M),length.out=200) #subset sequence
			lines(standardLowess$x[ii.s],standardLowess$y[ii.s],lwd=2,col="green") # adding the standard lowess line!
			abline(h=global.m,col="yellow",lwd=2)
			legend("bottomleft",c("Lowess","popLowess","globalMedian"),bty="n",xjust=1,cex=0.7,col=c("green","red","yellow"),lty=1,lwd=2)
			dev.off()
			rm(col.f,ii.s)
		}else{
			# global median adjustment to the largest population!
			clusterData<-lz[[which.max(dd$size)]]
			pop.corr<-0-median(clusterData[,4],na.rm=TRUE)
			rm(clusterData)
			# correct all data
			M.popLowess<-M+pop.corr
			gc();
		}#end global median
		gc();	
		
			
		## Final genome plot with adjustments for zlowess + peak selection
		myfile<-paste(assayNames[i],"_wholeGenomeFinished.ps",sep="")
		postscript(myfile)
		col.f<-rep("black",length(posOrder))
		jj<-sort(dd$size,index.return=TRUE,decreasing=TRUE)
		for(h in nbrClusters:1){
			clusterData<-lz[[jj$ix[h]]] #matrix, cols = segmentvalues, positions (identifier), A, M
			ii<-match(clusterData[,2],posOrder)
			col.f[ii]<-my.col[h]
		}
		plot(M.popLowess,cex=0.25,las=1,ylab="M",cex.axis=0.7,cex.lab=0.7,yaxt="n",col=col.f,xaxt="n",xlab="Chromosome")
		title(main=paste("Whole genome after zlowess + peak with sub pop for ",assayNames[i]),cex.main=0.8)
		abline(h=c(-0.5,0.5,1,-1,-1.5,1.5,0.3,-0.3),lty=3)
		abline(h=0)
		axis(side=2,at=seq(round(min(M.popLowess,na.rm=TRUE)),round(max(M.popLowess,na.rm=TRUE)),by=0.5),las=1,labels=seq(round(min(M.popLowess,na.rm=TRUE)),round(max(M.popLowess,na.rm=TRUE)),by=0.5),cex.axis=0.7)
		# Plot the vertical lines at chromosome boundries and create the values for the x-axis
		bound.index<-c(1,chr.MaxPosOrder) #add a first line
		abline(v=bound.index, lty = 1,col="black")
			
		# Arrange the x-axis with the chromosome names
		at.tics1<-matrix(ncol=length(chromosomes)/2,nrow=1,NA)
		at.tics2<-matrix(ncol=length(chromosomes)/2,nrow=1,NA)
		as.lab1<-matrix(ncol=length(chromosomes)/2,nrow=1,"")
		as.lab2<-matrix(ncol=length(chromosomes)/2,nrow=1,"")
		for(b in 1:length(chromosomes)){
			if(b%%2){
				at.tics1[b]<-mean(c(bound.index[b],bound.index[b+1]))
				as.lab1[b]<-chromosomes[b]
			}else{
				at.tics2[b]<-mean(c(bound.index[b],bound.index[b+1]))
				as.lab2[b]<-chromosomes[b]
			}
		}
		axis(side=1,labels=as.lab1,at=at.tics1,tick=TRUE,cex.axis=0.7,las=1,font=2,lty=0) #bottom sample name axis
		axis(side=3,labels=as.lab2,at=at.tics2,tick=TRUE,cex.axis=0.7,las=1,font=2,lty=0) #bottom sample name axis
		rm(at.tics1,at.tics2,as.lab1,as.lab2,bound.index)
		gc();
		dev.off()	
	
	
		### Adjust the real data (all)
		#keep A fixed = data$l10intgmean1_2
		#log10(sqrt(data$intensity1*data$intensity2))
		# r.vec = data$intensity1/data$intensity2
		# ch2= (10^A) /sqrt(r)
		# ch1= r*ch2
		r.vec<-2^(M.popLowess) #new ratio
	
		int2<-(10^l10intgmean1_2)/sqrt(r.vec)
		int1<-r.vec*int2
			
		## set the normalized values
		mod.int.matrix[,loop.control[i]]<-int1
		mod.int.matrix[,loop.control[i]+1]<-int2
		##
		rm(int1,int2,r.vec,M,l10intgmean1_2,dd,M.popLowess,col.f)
		gc();
	}#end for each assay
	gc();
	return(mod.int.matrix)
}#end popLowess function

