########################### MAIN ###############################
# NOTE: the R package DNAcopy is required!
#


###### Parameters to adjust

#1. File name/path to the input file. File should be tab separated
# [reporterId] [chromosome] [startPosition] [endPosition] [int1 sample1] 
# [int2 sample1] [int1 sample2] ...
# first four header columns should be:
# reporterId  chromosome  startPosition  endPosition

myfile<-"standAloneData.txt"


#2. Normalisation method. Use either 'lowess' or 'peak' (performs a global
# peak adjustment)

normMethod="lowess"


#3. Assay names. Assay names should if desired be present in a vector fed to
# the popLowess function. One entry per sample in the same order as the samples
# appear in the intensity matrix.

my.assaynames<-c("Xeno4_cy3","Xeno4_cy5")


#4. Significance threshold for accepting change points. CBS parameter

alpha<-0.05


#5. Several other parameters are present. For changing these, refer to the
# popLowess function directly.

#######


#### load the popLowess function
source("popLowessFunction.R")
####


#### Reading in data, sorting
data<-read.delim(myfile,header=TRUE)
colnames(data)[1:4]<-c("reporterId","chromosome","startPosition","endPosition")
data$chromosome<-as.character(data$chromosome)
data$chromosome[which(data$chromosome == "Y")]<-24 #x=23,y=24
data$chromosome[which(data$chromosome == "X")]<-23
data$chromosome<-as.integer(data$chromosome)
data$reporterId<-as.character(data$reporterId)	

ii<-order(data$chromosome,data$startPosition)
data<-data[ii,] #reorder all data according to chromosom and then to start kb
rm(ii)
gc();
####	


intensity.matrix<-as.matrix(data[,-c(1,2,3,4)])

library(DNAcopy)  #Needs to be loaded!

norm.int.matrix<-popLowess(data$reporterId,data$chromosome,data$startPosition,data$endPosition,intensity.matrix,normMethod="lowess",assayNames=my.assaynames)

## Write output
c.names<-colnames(data)
write.table(cbind(data[,1:4],norm.int.matrix), file = paste("Normalized_",myfile,sep=""), quote = FALSE,append=FALSE, sep = "\t", row.names = FALSE,col.names = c.names)
##